searchData={"items":[{"type":"module","title":"Peep","doc":"`Telemetry.Metrics` reporter supporting Statsd and Prometheus.\n\nTo use it, start the reporter with `start_link/1`, providing a list of\n`Telemetry.Metrics` metric definitions in a Keyword list matching the schema\nspecified in `Peep.Options`:\n\n    import Telemetry.Metrics\n\n    Peep.start_link([\n      name: :my_peep,\n      metrics: [\n        counter(\"http.request.count\"),\n        sum(\"http.request.payload_size\"),\n        last_value(\"vm.memory.total\"),\n        distribution(\"http.request.latency\")\n      ]\n    ])\n\nTo emit Statsd metrics, `Peep` supports both UDP and Unix Domain Sockets.","ref":"Peep.html"},{"type":"module","title":"Why another `Telemetry.Metrics` library? - Peep","doc":"Both `TelemetryMetricsStatsd` and `TelemetryMetrics.Prometheus` are great\nchoices for emitting telemetry. However, `Peep` makes several different\nchoices that may not be as general-purpose as either of those libraries.","ref":"Peep.html#module-why-another-telemetry-metrics-library"},{"type":"module","title":"No sampled metrics - Peep","doc":"Sampling is a popular approach to reduce the amount of Statsd data flowing out\nof a service, but naive sampling dramatically reduces visibility into the\nshapes of distributions. `Peep` represents distributions using histograms,\nusing a small exponential function by default. This sacrifices some\ngranularity on individual samples, but one usually doesn't mind too much if a\nsample value of `95` is rounded to `100`, or if `950` is rounded to `1000`.\nThese histograms are emitted to statsd using the optional sample rate of\n`1/$count`.\n\n`Peep` uses `:atomics` stored in `:ets` for performance. New `:atomics` arrays\nare created when a metric with a new set of tags is observed, so there is a\nslight overhead when handling the first telemetry event with a distinct set of\ntags. `Peep` reporter processes are not involved in the handling of any\n`:telemetry` events, so there's no chance of a single process becoming a\nbottleneck.","ref":"Peep.html#module-no-sampled-metrics"},{"type":"module","title":"Distributions are aggregated immediately - Peep","doc":"This is a consequence of choosing to represent distributions as\nhistograms. There is no step in `Peep`'s processing where samples are\naggregated in large batches. This leads to a flatter performance profile when\nscaling up.","ref":"Peep.html#module-distributions-are-aggregated-immediately"},{"type":"module","title":"Statsd packets are stuffed - Peep","doc":"This is something that is not (at the time of writing) supported by\n`TelemetryMetricsStatsd`, but the need for stuffing packets became pretty\nclear when handling tens of thousands of telemetry events every second. During\neach reporting period, metrics collected by a `Peep` reporter will be\ncollected into a minimum number of packets. Users can set the maximum packet\nsize in `Peep.Options`.","ref":"Peep.html#module-statsd-packets-are-stuffed"},{"type":"module","title":"Supported `:reporter_options` - Peep","doc":"- `:prometheus_type` - when using `sum/2` or `last_value/2` you can use this\n  option to define Prometheus' type used by such metric. By default `sum/2`\n  uses `counter` and `last_value/2` uses `gauge`. It can be useful when some\n  values are already precomputed, for example presummed socket stats.","ref":"Peep.html#module-supported-reporter_options"},{"type":"function","title":"Peep.allow_metric?/1","doc":"","ref":"Peep.html#allow_metric?/1"},{"type":"function","title":"Peep.assign_metric_ids/1","doc":"","ref":"Peep.html#assign_metric_ids/1"},{"type":"function","title":"Peep.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Peep.html#child_spec/1"},{"type":"function","title":"Peep.get_all_metrics/1","doc":"Fetches all metrics from the worker. Called when preparing Prometheus or\nStatsD data.","ref":"Peep.html#get_all_metrics/1"},{"type":"function","title":"Peep.get_metric/3","doc":"Fetches a single metric from storage. Currently only used in tests.","ref":"Peep.html#get_metric/3"},{"type":"function","title":"Peep.insert_metric/4","doc":"","ref":"Peep.html#insert_metric/4"},{"type":"function","title":"Peep.prune_tags/2","doc":"Removes metrics whose metadata contains the specified tag patterns.\n\nExample inputs:\n\n- `[%{foo: :bar}, %{baz: :quux}]` removes metrics with `foo == :bar` OR `baz == :quux`\n- `[%{foo: :bar, baz: :quux}]` removes metrics with `foo == :bar` AND `baz == :quux`\n- `[%{foo: :one}, %{foo: :two}]` removes metrics with `foo == :one` OR `foo == :two`","ref":"Peep.html#prune_tags/2"},{"type":"function","title":"Peep.start_link/1","doc":"","ref":"Peep.html#start_link/1"},{"type":"function","title":"Peep.storage_size/1","doc":"Returns measurements about the size of a running Peep's storage, in number of\nETS elements and in bytes of memory.","ref":"Peep.html#storage_size/1"},{"type":"type","title":"Peep.metric_id/0","doc":"","ref":"Peep.html#t:metric_id/0"},{"type":"module","title":"Peep.Options","doc":"Options for a `Peep` reporter. Validated with `NimbleOptions`.\n\n* `:name` (`t:atom/0`) - Required. A name for the Peep worker process\n\n* `:metrics` (list of `t:term/0`) - Required. A list of `Telemetry.Metrics` metric definitions to be collected and exposed.\n\n* `:statsd` - An optional keyword list of statsd configuration.\n\n    * `:flush_interval_ms` (`t:non_neg_integer/0`) - Time interval between StatsD metric emissions. The default value is `5000`.\n\n    * `:host` - Hostname or IP address of the StatsD server. The default value is `{127, 0, 0, 1}`.\n\n    * `:port` (`t:non_neg_integer/0`) - Port of the StatsD server. The default value is `8125`.\n\n    * `:socket_path` - Path to the Unix Domain Socket used for publishing instead of the hostname and port. Overrides `:host` and `:port` configuration if present\n\n    * `:formatter` - Determines the format of the published metrics. Can be either `:standard` or `:datadog`. The default value is `:standard`.\n\n    * `:mtu` (`t:non_neg_integer/0`) - Determine max size of statsd packets. For UDP, 1472 is a good choice. For UDS, 8192 is probably better.\n        The rationale for these recommendations comes from [this guide](https://docs.datadoghq.com/developers/dogstatsd/high_throughput/#ensure-proper-packet-sizes). The default value is `1472`.\n\n* `:global_tags` (`t:map/0`) - Additional tags published with every metric. Global tags are overriden by the tags specified in the metric definition. The default value is `%{}`.\n\n* `:storage` - Which storage implementation to use. `:default` uses a single ETS table, with some optimizations for concurrent writing. `:striped` uses one ETS table per scheduler thread, which trades memory for less lock contention for concurrent writes. The default value is `:default`.","ref":"Peep.Options.html"},{"type":"function","title":"Peep.Options.docs/0","doc":"","ref":"Peep.Options.html#docs/0"},{"type":"function","title":"Peep.Options.validate/1","doc":"","ref":"Peep.Options.html#validate/1"},{"type":"type","title":"Peep.Options.t/0","doc":"","ref":"Peep.Options.html#t:t/0"},{"type":"module","title":"Peep.Plug","doc":"Use this plug to expose your metrics on an endpoint for scraping. This is useful if you are using Prometheus.\n\nThis plug accepts the following options:\n\n* `:peep_worker` - The name of the Peep worker to use. This is required.\n* `:path` - The path to expose the metrics on. Defaults to `\"/metrics\"`.\n* `:on_unmatched_path` - The intended behavior of this plug when handling a\n  request to a different path. There are two possible values:\n    - `:continue` (default) - This allows for subsequent Plugs in a router to\n      be executed. This option is useful when Peep.Plug is part of a router\n      for a Phoenix application, or a router that matches other paths after\n      Peep.Plug.\n\n    - `:halt` - Responds to requests with 404. This option is useful when\n      Peep.Plug is used for serving metrics on a separate port, which is a\n      practice that is encouraged by other libraries that export Prometheus\n      metrics.","ref":"Peep.Plug.html"},{"type":"module","title":"Usage - Peep.Plug","doc":"You can use this plug in your Phoenix endpoint like this:\n\n  ```elixir\n  plug Peep.Plug, peep_worker: :my_peep_worker\n  ```\n\nOr if you'd rather use a different path:\n\n  ```elixir\n  plug Peep.Plug, path: \"/my-metrics\"\n  ```\n\nIf you are not using Phoenix, you can use it directly with Cowboy by adding this to your applications's supervision tree:\n\n  ```elixir\n  {Plug.Cowboy, scheme: :http, plug: Peep.Plug, options: [port: 9000]}\n  ```\n\nSimilarly, if you are using Bandit, you can use it like so:\n\n  ```elixir\n  {Bandit, [\n    scheme: :http,\n    plug: {Peep.Plug, peep_worker: :my_app},\n    port: 9000\n  ]}\n  ```","ref":"Peep.Plug.html#module-usage"},{"type":"module","title":"Peep.Prometheus","doc":"Prometheus exporter module.\n\nIf your application handles calls to \"GET /metrics\", your handler can call:\n\n    Peep.get_all_metrics(:my_peep) # Replace with your Peep reporter name\n    |> Peep.Prometheus.export()","ref":"Peep.Prometheus.html"},{"type":"function","title":"Peep.Prometheus.export/1","doc":"","ref":"Peep.Prometheus.html#export/1"},{"type":"module","title":"Peep.Telemetry","doc":"`:telemetry` events for `Peep` itself.","ref":"Peep.Telemetry.html"},{"type":"module","title":"Telemetry events - Peep.Telemetry","doc":"- `[:peep, :packet, :sent]`. Metadata contains `%{size: packet_size}`\n- `[:peep, :packet, :error]`. Metadata contains `%{reason: reason}`","ref":"Peep.Telemetry.html#module-telemetry-events"},{"type":"function","title":"Peep.Telemetry.sent_packet/2","doc":"","ref":"Peep.Telemetry.html#sent_packet/2"},{"type":"behaviour","title":"Peep.Storage","doc":"Behaviour for Peep storage backends. These functions are mainly called by Peep\nduring normal functioning. Ordinary usage of Peep should not require calling\nany of these functions.","ref":"Peep.Storage.html"},{"type":"callback","title":"Peep.Storage.get_all_metrics/2","doc":"Retrieves all stored metrics","ref":"Peep.Storage.html#c:get_all_metrics/2"},{"type":"callback","title":"Peep.Storage.get_metric/4","doc":"Retrieves a single stored metric","ref":"Peep.Storage.html#c:get_metric/4"},{"type":"callback","title":"Peep.Storage.insert_metric/5","doc":"Stores a sample metric","ref":"Peep.Storage.html#c:insert_metric/5"},{"type":"callback","title":"Peep.Storage.new/0","doc":"Creates a new term representing a Peep storage backend.","ref":"Peep.Storage.html#c:new/0"},{"type":"callback","title":"Peep.Storage.prune_tags/2","doc":"Removes metrics whose metadata contains a specific tag key and value.\nThis is intended to improve situations where Peep emits metrics whose tags\nhave high cardinality.","ref":"Peep.Storage.html#c:prune_tags/2"},{"type":"callback","title":"Peep.Storage.storage_size/1","doc":"Calculates the amount of memory used by a Peep storage backend.","ref":"Peep.Storage.html#c:storage_size/1"},{"type":"module","title":"Peep.Storage.ETS","doc":"Peep.Storage implementation using a single ETS table.\n\nA sane default for storing Peep metrics, with some simple optimizations.\nIf you discover that lock contention on Peep's ETS table is high,\nconsider switching to `Peep.Storage.Striped`, which reduces lock contention\nat the cost of higher memory usage.","ref":"Peep.Storage.ETS.html"},{"type":"module","title":"Peep.Storage.Striped","doc":"Peep.Storage implementation using an ETS table per BEAM scheduler thread.\n\nOffers less lock contention than `Peep.Storage.ETS`, at the cost of higher\nmemory usage. Recommended when executing thousands of metrics per second.","ref":"Peep.Storage.Striped.html"},{"type":"behaviour","title":"Peep.Buckets","doc":"A behavior for histogram bucketing strategies.\n\nYou can pass a bucketing strategy to a distribution metric in your metrics:\n\n```elixir\ndef metrics do\n  [\n    distribution([:bandit, :request, :stop, :duration],\n      tags: [],\n      unit: {:native, :millisecond},\n      reporter_options: [\n        peep_bucket_calculator: Peep.Buckets.PowersOfTen\n      ]\n    )\n  ]\nend\n```\n\nIf no bucketing strategy is provided is not set in :reporter_options for a\n`%Telemetry.Metrics.Distribution{}`, then the default is\n`Peep.Buckets.Exponential`.\n\nYou can change the default bucket calculator, set `:bucket_calculator` in your\nconfig.\n\n```elixir\nconfig :peep, bucket_calculator: Peep.Buckets.PowersOfTen\n```","ref":"Peep.Buckets.html"},{"type":"behaviour","title":"Custom Buckets - Peep.Buckets","doc":"If you want custom bucket boundaries, there is `Peep.Buckets.Custom`, which\nuses pattern matching to assign sample measurements to buckets.\n\nExample:\n\n```elixir\ndefmodule MyApp.MyBucket do\n  use Peep.Buckets.Custom,\n    buckets: [10, 100, 1_000]\nend\n```","ref":"Peep.Buckets.html#module-custom-buckets"},{"type":"callback","title":"Peep.Buckets.bucket_for/2","doc":"","ref":"Peep.Buckets.html#c:bucket_for/2"},{"type":"function","title":"Peep.Buckets.config/1","doc":"","ref":"Peep.Buckets.html#config/1"},{"type":"callback","title":"Peep.Buckets.config/1","doc":"","ref":"Peep.Buckets.html#c:config/1"},{"type":"callback","title":"Peep.Buckets.number_of_buckets/1","doc":"","ref":"Peep.Buckets.html#c:number_of_buckets/1"},{"type":"callback","title":"Peep.Buckets.upper_bound/2","doc":"","ref":"Peep.Buckets.html#c:upper_bound/2"},{"type":"type","title":"Peep.Buckets.config/0","doc":"","ref":"Peep.Buckets.html#t:config/0"},{"type":"module","title":"Peep.Buckets.Custom","doc":"A helper module for writing modules that implement the `Peep.Buckets`\nbehavior, with custom bucket boundaries.\n\nFor an example, look at the source of `Peep.Buckets.PowersOfTen`.","ref":"Peep.Buckets.Custom.html"},{"type":"module","title":"Peep.Buckets.Exponential","doc":"The default bucketing strategy in Peep, based on the logarithmic bucketing\nscheme in the DDSketch paper.\n\nThis bucketing scheme takes two parameters:\n\n1. `max_value` - a maximum expected value. Defaults to 1000000000.\n2. `bucket_variability` - a percentage reflecting roughly half the amount by\nwhich bucket ranges should grow. Defaults to 0.1.","ref":"Peep.Buckets.Exponential.html"},{"type":"module","title":"Peep.Buckets.PowersOfTen","doc":"An implementation of `Peep.Buckets`, using `Peep.Buckets.Custom`.","ref":"Peep.Buckets.PowersOfTen.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}